#!/usr/bin/perl

#Murasaki - multiple genome global alignment program
#Copyright (C) 2006-2007 Keio University
#(Kris Popendorf) <comp@bio.keio.ac.jp> (2006)
#
#This program is free software; you can redistribute it and/or
#modify it under the terms of the GNU General Public License
#as published by the Free Software Foundation; either version 2
#of the License, or (at your option) any later version.
#
#This program is distributed in the hope that it will be useful,
#but WITHOUT ANY WARRANTY; without even the implied warranty of
#MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#GNU General Public License for more details.
#
#You should have received a copy of the GNU General Public License
#along with this program; if not, write to the Free Software
#Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

use Getopt::Long;
use Pod::Usage;
use File::Basename;
use IO::Handle;
#use Data::Dump qw {dump};

#use Math::BigRat;

use strict;

our $root='/home/krisp/murasaki';
our $seqhome="$root/seq";

our ($help,$man);
GetOptions('help|?' => \$help, man => \$man);
pod2usage(1) if $help or $#ARGV<0;
pod2usage(-exitstatus => 0, -verbose => 2) if $man;

my $outfile=shift @ARGV;
if($outfile=~m/\.seqs$/){ #alignment file
  my $basename=$`;
  print "Loading sequence list from $outfile\n";
  open(INF,$outfile);
  while(<INF>){
    chomp;
    push(@ARGV,$_);
  }
  close(INF);
  $outfile="$basename.cdsaligned";
  print "Storing output to $outfile\n";
}

our (@seqcds,%names);

my $i;
LoadSeqs: foreach my $seq (@ARGV){
  if(!-f "$seq.cds"){
    print "CDS file not found for $seq. Generating...\n";
    my $res=system("$root/getcds.pl $seq");
    do {
      print "Generation of CDS file for $seq failed" unless -f "$seq.cds\n";
      push(@seqcds,undef);
      next LoadSeqs;
    } unless -f "$seq.cds";
  }
  open(CDS,"$seq.cds") or die "Couldn't read $seq.cds";
  print "Loading annotation for $seq...";
  my @cds;
 LoadCDS: while(<CDS>){
    my ($name,$start,$stop,$strand,$locus) = split(/\s/,$_);
    my $cd={name => $name,start=>$start,stop=>$stop,strand=>$strand,locus=>$locus};
    next unless $name;
      push(@cds,$cd);
    ${$names{$name}}[$i]=$cd;
  }
  print(($#cds+1)," CDS's loaded.\n");
  push(@seqcds,\@cds);
}continue{$i++}

open(OUTF,">$outfile") or die "Couldn't open $outfile for writing\n";
my $count=0;
foreach my $cds (@{$seqcds[0]}){
  do{
    print OUTF join("\t",map {(@{$_}{qw{start stop}},$_->{strand}<0 ? "-":"+")}
		    @{$names{$cds->{name}}})."\n";
    $count++;
  }
    if hasAll($cds->{name});
}
close OUTF;
print "Done. $count psuedo-anchors.\n";

sub hasAll {
  my $name=pop;
  for(my $i=0;$i<@seqcds;$i++){
    return 0 unless $names{$name}[$i];
  }
  return 1;
}

__END__

=head1 NAME

cdsalign.pl -- produce (untested) alignments from cds files

=head1 SYNOPSIS

cdalign.pl <output> <input1> <input2> [<input3> ...]

=head1 OPTIONS

=cut
